///	\file			MFCDevice.h
///	\date			2020
///	\copyright (c)	Metrolab Technology S.A., Geneva, Switzerland (www.metrolab.com)
///	See the included file License Terms.pdf for the licensing conditions.

#pragma once

// Standard includes
#include <string>
// Personal includes
#include "VISAInstrument.h"
#include "Helpers.h"
#include "InstrumentCatalog.h"

using namespace MTL;
using namespace MTL::Instrument;

namespace MTL {

	//--------------------------------------------------------------------------//
	//								MFC Device
	//--------------------------------------------------------------------------//

	class CMFCDevice
	{
		//----------------------------------------------------------------------//
		//								Types									//
		//----------------------------------------------------------------------//
	public:
		struct sMFCDeviceError {
			enum eLevel {
				kFatal		= 1,	// Fatal error. Should not occur, program may crash
				kError		= 2,	// Error. The requested operation cannot keep executing
				kWarning	= 3,	// Warning. An error occured but requested operation will continue to execute
				kNoError	= 4		// No error
			}				Level;
			std::string		Description;
			std::string		Location;
			sMFCDeviceError()
				: Level(kError), Description(""), Location("")
			{}
			sMFCDeviceError(eLevel ErrLevel, std::string ErrDesc = "", std::string Loc = "")
				: Level(ErrLevel), Description(ErrDesc), Location(Loc)
			{}
			bool HasError()
			{
				return (kNoError != Level);
			}
			operator bool() const // return true if no Error!
			{
				return (kNoError == Level);
			}
		};

		//--------------------------------------------------------------------------//
		//							Constructor / Destructor
		//--------------------------------------------------------------------------//
	public:
		virtual ~CMFCDevice();

		//--------------------------------------------------------------------------//
		//							Utilities
		//--------------------------------------------------------------------------//
	public:
		bool Is(const CMFCDevice * pDevice);
		virtual bool AbortInstrument() = 0;
		virtual bool IsInstrumentConnected() =0 ;

		//--------------------------------------------------------------------------//
		//							Available Operations
		//--------------------------------------------------------------------------//
	public:
		virtual sMFCDeviceError Connect(	CInstrumentCatalog::sInstrumentInformation & rInstrInfo) = 0;
		virtual sMFCDeviceError ScanPA(	CInstrumentCatalog::sProbeArrayInformation &	rPAInfo,
										CInstrumentCatalog::sInstrumentConfiguration & rInstrConfig, bool &rPaInfoChanged) = 0;
		virtual sMFCDeviceError Search(const CInstrumentCatalog::sInstrumentState & rInstrState) = 0;
		virtual sMFCDeviceError Measure(const CInstrumentCatalog::sInstrumentState & rInstrState) = 0;
		virtual sMFCDeviceError NewNormalizationRequest(	const CInstrumentCatalog::sNormalization & rNewNormalizationRequest,
															CInstrumentCatalog::sNormalization & rReturnedNormalization) = 0;
		virtual sMFCDeviceError GetNormalizationDate(CInstrumentCatalog::tISO8601DateTime & rNormDate)=0;
		virtual sMFCDeviceError SetRBBusyLedState(const CInstrumentCatalog::eRemoteBoxBusyLedState & rRBBusyLedState) = 0;
		virtual sMFCDeviceError RemoteBoxButtonStatus(bool & rRemoteStartPressed) = 0;
		virtual sMFCDeviceError ChangeSerialParmsOnConnectedInstr(const CInstrumentCatalog::sMFC3045SerialPortSettings & rNewSerialSetting) = 0;

	}; // class

} // namespace MTL
