///	\file			MFCManagerCatalog.h
///	\date			2020
///	\copyright (c)	Metrolab Technology S.A., Geneva, Switzerland (www.metrolab.com)
///	See the included file License Terms.pdf for the licensing conditions.

#pragma once

#include "CatalogInterface.h"		// Catalog definitions
#include "VISAInstrument.h"			// CVISAResourceManager & CParsedResourceList
#include "OSDefines.h"				// Basic types U8, U16 etc...
#include "InstrumentCatalog.h"		// CInstrumentCatalog types
#include "MFCDevice.h"

using namespace MTL;
using namespace MTL::InstrumentCatalogInterface;
using namespace MTL::Instrument;

namespace MTL {
	class CMFCManagerCatalog {
	public:
		//----------------------------------------------------------------------//
		//								Types									//
		//----------------------------------------------------------------------//
		enum class eControllerState { kIdle, kScanningInstruments, kScanningPA, kSearching, kMeasuring };

		friend class CMFCManagerInterface;
		friend class CMFCControllerInterface;
		friend class CUSBRemoteBoxInterface;

	protected:
		//----------------------------------------------------------------------//
		//								Variables								//
		//----------------------------------------------------------------------//
		//------------------------------------------//
		// MFCController Results
		CVariable<CMFCManagerCatalog::eControllerState>			vState;
		CVariable<CParsedResourceList>							vInstrumentList;
		CVariable<CInstrumentCatalog::sProbeArrayInformation>	vPAInfo;
		CVariable<CInstrumentCatalog::sInstrumentConfiguration>	vInstrumentConfiguration;
		CVariable<U8>											vSearchProgress;
		CVariable<CInstrumentCatalog::sSearchResult>			vSearchResult;
		CVariable<CInstrumentCatalog::sMeasurementResults>		vMeasResults;
		CVariable<CMFCDevice::sMFCDeviceError>					vError;
		CVariable<CInstrumentCatalog::sLogMessage>				vLogMessage;
		CVariable<bool>											vRemoteBoxStart;
		//------------------------------------------//
		// USBRemoteBoxController Control
		CVariable<bool>											vUSBRemoteBusyState;
		//------------------------------------------//
		// USBRemoteBoxController Results
		CVariable<bool>											vUSBRemoteBoxStart;
		CVariable<bool>											vUSBRemoteBoxReset;
	}; // class CInstrumentCatalog

	class CMFCManagerInterface
	{
	public:
		//----------------------------------------------------------------------//
		//							Controls and Observers						//
		//----------------------------------------------------------------------//
		//------------------------------------------//
		// MFCController Results
		CObserver<CMFCManagerCatalog::eControllerState>			oState;
		CObserver<CParsedResourceList>							oInstrumentList;
		CObserver<CInstrumentCatalog::sProbeArrayInformation>	oPAInfo;
		CObserver<CInstrumentCatalog::sInstrumentConfiguration>	oInstrumentConfiguration;
		CObserver<U8>											oSearchProgress;
		CObserver<CInstrumentCatalog::sSearchResult>			oSearchResult;
		CObserver<CInstrumentCatalog::sMeasurementResults>		oMeasResults;
		CObserver<CMFCDevice::sMFCDeviceError>					oError;
		CObserver<CInstrumentCatalog::sLogMessage>				oLogMessage;
		CObserver<bool>											oRemoteBoxStart;
		//------------------------------------------//
		// USBRemoteBoxController Control
		CControl<bool>											cUSBRemoteBusyState;
		//------------------------------------------//
		// USBRemoteBoxController Results
		CObserver<bool>											oUSBRemoteBoxStart;
		CObserver<bool>											oUSBRemoteBoxReset;

		CMFCManagerInterface(CMFCManagerCatalog & rCatalog)
			: oState(rCatalog.vState),
			oInstrumentList(rCatalog.vInstrumentList),
			oPAInfo(rCatalog.vPAInfo),
			oInstrumentConfiguration(rCatalog.vInstrumentConfiguration),
			oSearchProgress(rCatalog.vSearchProgress),
			oSearchResult(rCatalog.vSearchResult),
			oMeasResults(rCatalog.vMeasResults),
			oError(rCatalog.vError),
			oLogMessage(rCatalog.vLogMessage),
			oRemoteBoxStart(rCatalog.vRemoteBoxStart),
			cUSBRemoteBusyState(rCatalog.vUSBRemoteBusyState),
			oUSBRemoteBoxStart(rCatalog.vUSBRemoteBoxStart),
			oUSBRemoteBoxReset(rCatalog.vUSBRemoteBoxReset)
		{}
	};

	class CMFCControllerInterface
	{
	public:
		//----------------------------------------------------------------------//
		//							Controls and Observers						//
		//----------------------------------------------------------------------//
		//------------------------------------------//
		// MFCController Results
		CControl<CMFCManagerCatalog::eControllerState>			cState;
		CControl<CParsedResourceList>							cInstrumentList;
		CControl<CInstrumentCatalog::sProbeArrayInformation>	cPAInfo;
		CControl<CInstrumentCatalog::sInstrumentConfiguration>	cInstrumentConfiguration;
		CControl<U8>											cSearchProgress;
		CControl<CInstrumentCatalog::sSearchResult>				cSearchResult;
		CControl<CInstrumentCatalog::sMeasurementResults>		cMeasResults;
		CControl<CMFCDevice::sMFCDeviceError>					cError;
		CControl<CInstrumentCatalog::sLogMessage>				cLogMessage;
		CControl<bool>											cRemoteBoxStart;

		CMFCControllerInterface(CMFCManagerCatalog & rCatalog)
			: cState(rCatalog.vState),
			cInstrumentList(rCatalog.vInstrumentList),
			cPAInfo(rCatalog.vPAInfo),
			cInstrumentConfiguration(rCatalog.vInstrumentConfiguration),
			cSearchProgress(rCatalog.vSearchProgress),
			cSearchResult(rCatalog.vSearchResult),
			cMeasResults(rCatalog.vMeasResults),
			cError(rCatalog.vError),
			cLogMessage(rCatalog.vLogMessage),
			cRemoteBoxStart(rCatalog.vRemoteBoxStart)
		{}
	};

	class CUSBRemoteBoxInterface
	{
	public:
		//----------------------------------------------------------------------//
		//							Controls and Observers						//
		//----------------------------------------------------------------------//
		//------------------------------------------//
		// USBRemoteBoxController Control
		CObserver<bool>										oUSBRemoteBusyState;
		//------------------------------------------//
		// USBRemoteBoxController Results
		CControl<bool>										cUSBRemoteBoxStart;
		CControl<bool>										cUSBRemoteBoxReset;

		CUSBRemoteBoxInterface(CMFCManagerCatalog & rCatalog)
			: oUSBRemoteBusyState(rCatalog.vUSBRemoteBusyState),
			cUSBRemoteBoxStart(rCatalog.vUSBRemoteBoxStart),
			cUSBRemoteBoxReset(rCatalog.vUSBRemoteBoxReset)
		{}
	};
} // namespace MTL
