#pragma once

#include "Synchronization.h"
#include <map>

namespace MTL {
	namespace InstrumentCatalogInterface {

		using namespace MTL::Synchronization;
		template <class DataType> class CControl;
		template <class DataType> class CObserver;
		template <class DataType> class CObserverNotifier;
		class CObserverNotifier_untyped;

		//----------------------------------------------------------------------//
		//							Catalog variable							//
		//----------------------------------------------------------------------//
		template <class DataType>
		class CVariable
		{
			friend class CControl<DataType>;
			friend class CObserver<DataType>;

			//------------------------------------------//
			//				Attributes					//
			//------------------------------------------//
		private:
			typedef CObserverNotifier_untyped *			tpObserverNotifier;		// Observer notifier pointer type
			typedef CObserver<DataType> *				tpObserver;				// Observer pointer type
			std::map<tpObserverNotifier, tpObserver>	m_Subscribers;			// List of notification subscribers
		protected:
			DataType			m_Value;		// Current value of the variable.
			CRecursiveMutex		m_Lock;			// Lock to the current object.

		public:
			//------------------------------------------//
			//		Constructors / Desctructors			//
			//------------------------------------------//
			CVariable()
			{}
			CVariable(DataType InitialValue)
				: m_Value(InitialValue)
			{}
			virtual ~CVariable()
			{}

		protected:
			//------------------------------------------//
			//				Value access				//
			//------------------------------------------//
			void rWrite(const DataType & rNewValue)
			{
				m_Lock.lock();
				m_Value = rNewValue;
				m_Lock.unlock();
				l_NotifyChange();
			}
			void Write(const DataType NewValue)
			{
				rWrite(NewValue);
			}
			void rRead(DataType & rReturnedValue)
			{
				m_Lock.lock();
				rReturnedValue = m_Value;
				m_Lock.unlock();
			}
			DataType Read(void)
			{
				DataType l_Value;
				rRead(l_Value);
				return l_Value;
			}

			//------------------------------------------//
			//				Event handling				//
			//------------------------------------------//
			void Subscribe(CObserverNotifier_untyped & rNotifier, CObserver<DataType> & rObs)
			{
				m_Lock.lock();
				m_Subscribers.insert(std::pair<tpObserverNotifier, tpObserver>(&rNotifier, &rObs));
				m_Lock.unlock();
			}
			void Unsubscribe(CObserverNotifier_untyped & rNotifier)
			{
				m_Lock.lock();
				m_Subscribers.erase(&rNotifier);	//TODO: Check return
				m_Lock.unlock();
			}
		private:
			void l_NotifyChange(void)
			{
				m_Lock.lock();
				// For all subscribers
				for (auto l_it = m_Subscribers.begin();
					l_it != m_Subscribers.end();
					l_it++)
				{
					// Notify!
					(l_it->first)->NotifyVarUpdate(*(l_it->second));
				}
				m_Lock.unlock();
			}
		};

	} // namespace InstrumentCatalogInterface;
} // namespace MTL
